#!/usr/bin/perl                                           -*- Perl -*-
#
# Copyright (c) 2002, 2003  FUJIWARA Teruyoshi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
use Jcode;
use FreePWING::FPWUtils::FPWParser;

my $log_code = 'euc';

$log_code = shift(@ARGV);
log_print("デバッグ出力の文字コードを $log_code に設定します．\n", 'utf8');

my ($fpwword2, $fpwheading, $fpwtext);
initialize_fpwparser('text'    => \$fpwtext,
		     'heading' => \$fpwheading,
		     'word2'   => \$fpwword2);

while(1) {
    last if (!defined($_ = <>));
    chomp;
    $_ =~ s/\r//;
    my ($word, $desc) = &parse_line(jcode($_, 'sjis')->utf8);
    my $title;

    if (length($word) > 63) {
	log_print("見出し語が長すぎます($word)．\n", 'utf8');
	next;
    }

    if ($word =~ s/^!//) {
	$title = jcode("$word [!]", 'utf8')->euc;
    } else {
	$title = jcode($word, 'utf8')->euc;
    }
    log_print("<$word><$desc>\n", 'utf8');

    ## 本文と見出しを新しいエントリに切り替える．
    $fpwtext->new_entry()
        || die $fpwtext->error_message() . "\n";
    $fpwheading->new_entry()
        || die $fpwheading->error_message() . "\n";
    
    ## 読み込んだ見出しを「本文部」に書き込む．
    if (!$fpwtext->add_keyword_start()
        || !$fpwtext->add_text($title)
        || !$fpwtext->add_keyword_end()
        || !$fpwtext->add_newline()) {
        die $fpwtext->error_message() . "\n";
     }

    ## 読み込んだ見出しを「見出し部」に書き込む．
    $fpwheading->add_text($title)
        || die $fpwheading->error_message() . "\n";
    
    ## 検索語を登録する．
    my @kw_array = &keyword($word);
    $heading_position = $fpwheading->entry_position();
    $text_position = $fpwtext->entry_position();

    foreach my $kw (@kw_array) {
	my $k = jcode($kw, 'utf8')->euc;
	# () が見出し語の後半についている語を誤認識して落ちる問題を回避．
	next if length $kw == 0;
	if (!$fpwword2->add_entry($k, $heading_position, $text_position)) {
	    log_print($k . ":" . $fpwword2->error_message() ."\n", 'euc');
	    next;
	}
    }
    
    ## 本文を書き込む．
    if (!$fpwtext->add_text(jcode($desc, 'utf8')->euc)
        || !$fpwtext->add_newline()) {
        die $fpwtext->error_message() . "\n";
    }
}

finalize_fpwparser('text'    => \$fpwtext,
		   'heading' => \$fpwheading,
		   'word2'   => \$fpwword2);

sub parse_line {
    my ($line) = @_;

    $line =~ s/^■//;
    my ($word, $desc) = split(" *: *", $line);

    return ($word, $desc);
}

sub keyword {
    my ($kw) = @_;
    my @result;
    
    $kw =~ s/；.*$//;
    $kw =~ s/〔.*〕//;
    $kw =~ s/ *{.*}//;
    $kw =~ s/（.*）$//;
    $kw =~ s/\t//;

    if ($kw =~ s/【(.*)】//) {
	@result = split("・", $1);
    }
    push(@result, $kw);
    
    return @result;
}

sub log_print
{
    my ($mes, $icode) = @_;
    
    &Jcode::convert(\$mes, $log_code, $icode);
    print $mes;
}
